/*
 *                   jfjji,.           H A M M E R H E A D  S H A R K W A R E
 *                  ,#######EL:                  http://www.hhsw.de
 *                   .,j######Kj       
 *                    ;W########G:     This  code  is  free software; you can
 *                   j#####G ,D#L:     redistribute it and/or modify it under
 *             .,;;,G#####D.           the  terms  of  the GNU General Public
 *           jE##########G             License   as  published  by  the  Free
 *             .,W######D.             Software  Foundation; either version 2
 *             :L#####K##G             of  the  License,  or (at your option)
 *           .f#####G;.t##:            any later version.
 *         ;L###ELi.    ,D,            
 *       iEEGWj                        This  code  is distributed in the hope
 *      tW;                            that  it  will  be useful, but without
 *     jf                              any warranty; without even the implied
 *    .Kt                              warranty of merchantability or fitness
 *    i:                               for  a particular purpose. See the GNU
 *    ;                                General   Public   License   for  more
 *    .                                details (http://www.gnu.org).
 */
#include <string>
#include <vector>

#include "Icon.hh"

using namespace std;
using namespace bbdock;

/**
 * Initialize icon
 *
 * @param i - filename of image file
 * @param c - command to execute
 * @param w - window title if application should run exclusively (optional)
 * @param ms - idle time in miliseconds (optional)
 */
Icon::Icon( string i, string c, string t, unsigned int ms ) :
	image( expand( i ) ),
	command( expand( c ) ),
	idletime( ms )
{
	setTitle( t );
}

/**
 * Shell expansion
 *
 * @param path - path to expand
 */
string Icon::expand( string path ) const
{
	if( path[0] == '~' )
		path = getenv( "HOME" )+path.substr( 1 );

	return path;
}

/**
 * Tokenize string into vector
 *
 * @param str - string to split
 * @param tokens - vector that receives the tokens
 * @param delimiter - delimiter characters
 */
void Icon::tokenize( const string str, vector<string> &tokens,
	const string& delimiters ) const
{
    string::size_type last = str.find_first_not_of( delimiters, 0 );
    string::size_type next = str.find_first_of( delimiters, last );

    while( next != string::npos || last != string::npos )
    {
		tokens.push_back( str.substr( last, next-last ) );

		last = str.find_first_not_of( delimiters, next );
		next = str.find_first_of( delimiters, last );
    }
}
