/*
 *                   jfjji,.           H A M M E R H E A D  S H A R K W A R E
 *                  ,#######EL:                  http://www.hhsw.de
 *                   .,j######Kj       
 *                    ;W########G:     This  code  is  free software; you can
 *                   j#####G ,D#L:     redistribute it and/or modify it under
 *             .,;;,G#####D.           the  terms  of  the GNU General Public
 *           jE##########G             License   as  published  by  the  Free
 *             .,W######D.             Software  Foundation; either version 2
 *             :L#####K##G             of  the  License,  or (at your option)
 *           .f#####G;.t##:            any later version.
 *         ;L###ELi.    ,D,            
 *       iEEGWj                        This  code  is distributed in the hope
 *      tW;                            that  it  will  be useful, but without
 *     jf                              any warranty; without even the implied
 *    .Kt                              warranty of merchantability or fitness
 *    i:                               for  a particular purpose. See the GNU
 *    ;                                General   Public   License   for  more
 *    .                                details (http://www.gnu.org).
 */
#ifndef bbdock_Render_hh
#define bbdock_Render_hh

#if defined( __GNUC__ ) || \
	defined( __i686__ ) || \
	defined( __i586__ ) || \
	defined( __i486__ ) || \
	defined( __i386__ )
#define bbdock_x86_att		1
#elif defined( _M_IX86 )
#define bbdock_x86_intel	1
#else
#include <math.h>
#endif

namespace bbdock
{
	/**
	 * Abstract class, this object is only a base for the following
	 * implentations, it got also a static member wich use is to invoke
	 * the right object
	 *
	 * @author mf@hhsw.de
	 * @version 0.1.4
	 */
	class Render
	{
		public:
			enum MarkType
			{
				PlayMark,
				DotsMark,
				CornerMark,
				CrossMark
			};

			inline const int &getSize() const { return size; }

			Render() {}
			virtual ~Render() {}
			Render &operator=( Render & ) {}
			static Render *getInstance( int, int, int );
			virtual void opaque( unsigned char *, unsigned char * ) {}
			virtual void ghosted( unsigned char *, unsigned char *, 
				unsigned char ) {}
			virtual void activeMark( unsigned char *, MarkType = PlayMark,
				int = 0, int = 0 ) {}

		protected:
			/**
			 * A mark
			 */
			class Mark
			{
				public:
					inline const int &getWidth() const { return width; }
					inline const int &getHeight() const { return height; }
					inline const char *getPixels() const { return pixels; }

					Mark( const int w, const int h, const char *p ) :
						width( w ),
						height( h ),
						pixels( p ) {}
					virtual ~Mark() {}

				private:
					const int width;
					const int height;
					const char *pixels;
			};

			int width;
			int height;
			int size;
			int padding;
			int bytesperline;
			int pixels;
			static const Render::Mark play;
			static const Render::Mark dots;
			static const Render::Mark corner;
			static const Render::Mark cross;

			virtual void initialize( int, int, int );
			virtual const Render::Mark &getMark( MarkType ) const;
			virtual unsigned int getOffset( int, int, Render::Mark &, 
				int ) const;
	};

	/**
	 * Render16 knows how to draw on surfaces of 16 bits color depth
	 *
	 * @author mf@hhsw.de
	 * @version 0.1.3
	 */
	class Render16 : Render
	{
		public:
			Render16( int, int );
			virtual ~Render16() {}
			virtual void opaque( unsigned char *, unsigned char * );
			virtual void ghosted( unsigned char *, unsigned char *, 
				unsigned char );
			virtual void activeMark( unsigned char *, MarkType = PlayMark,
				int = 0, int = 0 );
	};

	/**
	 * Render24 knows how to draw on surfaces of 24 bits color depth
	 *
	 * @author mf@hhsw.de
	 * @version 0.1.3
	 */
	class Render24 : Render
	{
		public:
			Render24( int, int );
			virtual ~Render24() {}
			virtual void opaque( unsigned char *, unsigned char * );
			virtual void ghosted( unsigned char *, unsigned char *, 
				unsigned char );
			virtual void activeMark( unsigned char *, MarkType = PlayMark,
				int = 0, int = 0 );
	};

	/**
	 * Render32 knows how to draw on surfaces of 32 bits color depth
	 *
	 * @author mf@hhsw.de
	 * @version 0.1.3
	 */
	class Render32 : Render
	{
		public:
			Render32( int, int );
			virtual ~Render32() {}
			virtual void opaque( unsigned char *, unsigned char * );
			virtual void ghosted( unsigned char *, unsigned char *, 
				unsigned char );
			virtual void activeMark( unsigned char *, MarkType = PlayMark,
				int = 0, int = 0 );
	};
}

#endif
